/*
 * This library is part of Geranium -
 * an open source UI library for GWT.
 *
 * Copyright (c) Alkacon Software GmbH (http://www.alkacon.com)-
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * For further information about Alkacon Software, please see the
 * company website: http://www.alkacon.com
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package com.alkacon.geranium.client.ui;

import com.alkacon.geranium.client.ui.css.I_LayoutBundle;

import com.google.gwt.core.client.GWT;
import com.google.gwt.dom.client.Element;
import com.google.gwt.uibinder.client.UiBinder;
import com.google.gwt.uibinder.client.UiField;
import com.google.gwt.user.client.ui.Widget;

/**
 * Displays a message text with an icon in front. Default is the 'warning' icon.<p>
 */
public class MessageWidget extends Widget {

    /** The UI binder interface. */
    interface I_CmsMessageWidgetUiBinder extends UiBinder<Element, MessageWidget> {
        // nothing to do
    }

    /** The UI binder instance. */
    private static I_CmsMessageWidgetUiBinder uiBinder = GWT.create(I_CmsMessageWidgetUiBinder.class);

    /** The element displaying the icon. */
    @UiField
    protected Element m_icon;

    /** The element holding the message. */
    @UiField
    protected Element m_message;

    /** The current icon CSS class. */
    private String m_iconClass;

    /**
     * Constructor.<p>
     */
    public MessageWidget() {

        setElement(uiBinder.createAndBindUi(this));
        setIconClass(I_LayoutBundle.INSTANCE.gwtImages().style().warningBigIcon());
    }

    /**
     * Sets the icon CSS class.<p>
     * 
     * @param iconClass the icon CSS class
     */
    public void setIconClass(String iconClass) {

        if (m_iconClass != null) {
            m_icon.removeClassName(m_iconClass);
        }
        m_iconClass = iconClass;
        m_icon.addClassName(m_iconClass);
    }

    /**
     * Sets the message HTML.<p>
     * 
     * @param message the message HTML
     */
    public void setMessageHtml(String message) {

        m_message.setInnerHTML(message);
    }

    /**
     * Sets the message text.<p>
     * 
     * @param message the message text
     */
    public void setMessageText(String message) {

        m_message.setInnerText(message);
    }
}
